@@ -53,7 +53,6 @@ static bool panel_utils_grab_available()
 
 	GdkWindow* root = gdk_screen_get_root_window(xfce_gdk_screen_get_active(NULL));
 	GdkDisplay* display = gdk_display_get_default();
-#if GTK_CHECK_VERSION(3,20,0)
 	GdkSeat* seat = gdk_display_get_default_seat(display);
 
 	// Don't try to get the grab for longer then 1/4 second
@@ -67,51 +66,6 @@ static bool panel_utils_grab_available()
 		}
 		g_usleep(100);
 	}
-#else
-	GdkDeviceManager* device_manager = gdk_display_get_device_manager(display);
-	GdkDevice* pointer = gdk_device_manager_get_client_pointer(device_manager);
-	GdkDevice* keyboard = gdk_device_get_associated_device(pointer);
-
-	// Don't try to get the grab for longer then 1/4 second
-	GdkGrabStatus grab_pointer = GDK_GRAB_FROZEN;
-	GdkGrabStatus grab_keyboard = GDK_GRAB_FROZEN;
-	for (guint i = 0; i < (G_USEC_PER_SEC / 400); ++i)
-	{
-		grab_keyboard = gdk_device_grab(keyboard,
-				root,
-				GDK_OWNERSHIP_NONE,
-				true,
-				GDK_ALL_EVENTS_MASK,
-				NULL,
-				GDK_CURRENT_TIME);
-		if (grab_keyboard == GDK_GRAB_SUCCESS)
-		{
-			grab_pointer = gdk_device_grab(pointer,
-					root,
-					GDK_OWNERSHIP_NONE,
-					true,
-					GDK_ALL_EVENTS_MASK,
-					NULL,
-					GDK_CURRENT_TIME);
-			if (grab_pointer == GDK_GRAB_SUCCESS)
-			{
-				grab_succeed = true;
-				break;
-			}
-		}
-		g_usleep(100);
-	}
-
-	// Release the grab so the menu window can take it
-	if (grab_pointer == GDK_GRAB_SUCCESS)
-	{
-		gdk_device_ungrab(pointer, GDK_CURRENT_TIME);
-	}
-	if (grab_keyboard == GDK_GRAB_SUCCESS)
-	{
-		gdk_device_ungrab(keyboard, GDK_CURRENT_TIME);
-	}
-#endif
 
 	if (!grab_succeed)
 	{
@@ -163,12 +117,6 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 	gtk_widget_set_name(m_button, "whiskermenu-button");
 #if !LIBXFCE4PANEL_CHECK_VERSION(4,13,0)
 	widget_add_css(m_button, ".xfce4-panel button { padding: 1px; }");
-	gtk_button_set_relief(GTK_BUTTON(m_button), GTK_RELIEF_NONE);
-#if GTK_CHECK_VERSION(3,20,0)
-	gtk_widget_set_focus_on_click(GTK_WIDGET(m_button), false);
-#else
-	gtk_button_set_focus_on_click(GTK_BUTTON(m_button), false);
-#endif
 #endif
 	g_signal_connect_slot(m_button, "toggled", &Plugin::button_toggled, this);
 	gtk_widget_show(m_button);
@@ -179,13 +127,13 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 	gtk_widget_show(GTK_WIDGET(m_button_box));
 
 	m_button_icon = GTK_IMAGE(gtk_image_new());
-	widget_add_css(GTK_WIDGET(m_button_icon), "image { padding: 3px; }");
 	icon_changed(wm_settings->button_icon_name.c_str());
 	gtk_box_pack_start(m_button_box, GTK_WIDGET(m_button_icon), true, false, 0);
 	if (wm_settings->button_icon_visible)
 	{
 		gtk_widget_show(GTK_WIDGET(m_button_icon));
 	}
+	gtk_widget_set_sensitive(GTK_WIDGET(m_button_icon), false);
 
 	m_button_label = GTK_LABEL(gtk_label_new(NULL));
 	gtk_label_set_markup(m_button_label, wm_settings->button_title.c_str());
@@ -194,6 +142,7 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 	{
 		gtk_widget_show(GTK_WIDGET(m_button_label));
 	}
+	gtk_widget_set_sensitive(GTK_WIDGET(m_button_label), false);
 
 	// Add plugin to panel
 	gtk_container_add(GTK_CONTAINER(plugin), m_button);
@@ -218,7 +167,7 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 	g_signal_connect_slot<GtkWidget*,GdkScreen*>(m_button, "screen-changed", &Plugin::update_size, this);
 
 	// Create menu window
-	m_window = new Window;
+	m_window = new Window(this);
 	g_signal_connect_slot<GtkWidget*>(m_window->get_widget(), "unmap", &Plugin::menu_hidden, this);
 }
 
@@ -338,6 +287,14 @@ void Plugin::set_configure_enabled(bool enabled)
 
 //-----------------------------------------------------------------------------
 
+void Plugin::set_loaded(bool loaded)
+{
+	gtk_widget_set_sensitive(GTK_WIDGET(m_button_icon), loaded);
+	gtk_widget_set_sensitive(GTK_WIDGET(m_button_label), loaded);
+}
+
+//-----------------------------------------------------------------------------
+
 void Plugin::button_toggled(GtkToggleButton* button)
 {
 	if (gtk_toggle_button_get_active(button) == false)
@@ -348,7 +305,7 @@ void Plugin::button_toggled(GtkToggleButton* button)
 	else
 	{
 		xfce_panel_plugin_block_autohide(m_plugin, true);
-		show_menu(m_button, xfce_panel_plugin_get_orientation(m_plugin) == GTK_ORIENTATION_HORIZONTAL);
+		show_menu(false);
 	}
 }
 
@@ -409,7 +366,7 @@ gboolean Plugin::remote_event(XfcePanelPlugin*, gchar* name, GValue* value)
 	}
 	else if (value && G_VALUE_HOLDS_BOOLEAN(value) && g_value_get_boolean(value))
 	{
-		show_menu(NULL, true);
+		show_menu(true);
 	}
 	else
 	{
@@ -468,26 +425,33 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 			0, GTK_PACK_START);
 
 	// Resize icon
+	if (wm_settings->button_single_row)
+	{
+		size /= xfce_panel_plugin_get_nrows(m_plugin);
+	}
 #if LIBXFCE4PANEL_CHECK_VERSION(4,13,0)
 	gint icon_size = xfce_panel_plugin_get_icon_size(m_plugin);
 #else
-	gint icon_size = size / xfce_panel_plugin_get_nrows(m_plugin);
-	icon_size -= 4;
-	if (icon_size < 24)
-	{
-		icon_size = 16;
-	}
-	else if (icon_size < 32)
-	{
-		icon_size = 24;
-	}
-	else if (icon_size < 36)
-	{
-		icon_size = 32;
-	}
+	GtkBorder padding, border;
+	GtkStyleContext* context = gtk_widget_get_style_context(m_button);
+	GtkStateFlags flags = gtk_widget_get_state_flags(m_button);
+	gtk_style_context_get_padding(context, flags, &padding);
+	gtk_style_context_get_border(context, flags, &border);
+	gint xthickness = padding.left + padding.right + border.left + border.right;
+	gint ythickness = padding.top + padding.bottom + border.top + border.bottom;
+	gint icon_size = size - 2 * std::max(xthickness, ythickness);
 #endif
 	gtk_image_set_pixel_size(m_button_icon, icon_size);
 
+	if (wm_settings->button_title_visible)
+	{
+		gtk_widget_set_size_request(m_button, -1, -1);
+	}
+	else
+	{
+		gtk_widget_set_size_request(m_button, size, size);
+	}
+
 	if (wm_settings->button_title_visible || !wm_settings->button_single_row)
 	{
 		xfce_panel_plugin_set_small(m_plugin, false);
@@ -532,19 +496,19 @@ void Plugin::update_size()
 
 //-----------------------------------------------------------------------------
 
-void Plugin::show_menu(GtkWidget* parent, bool horizontal)
+void Plugin::show_menu(bool at_cursor)
 {
 	if (wm_settings->menu_opacity != m_opacity)
 	{
 		if ((m_opacity == 100) || (wm_settings->menu_opacity == 100))
 		{
 			delete m_window;
-			m_window = new Window;
+			m_window = new Window(this);
 			g_signal_connect_slot<GtkWidget*>(m_window->get_widget(), "unmap", &Plugin::menu_hidden, this);
 		}
 		m_opacity = wm_settings->menu_opacity;
 	}
-	m_window->show(parent, horizontal);
+	m_window->show(at_cursor ? Window::PositionAtCursor : Window::Position(xfce_panel_plugin_get_orientation(m_plugin)));
 }
 
 //-----------------------------------------------------------------------------
