@@ -58,6 +58,7 @@ public:
 	void set_button_title(const std::string& title);
 	void set_button_icon_name(const std::string& icon);
 	void set_configure_enabled(bool enabled);
+	void set_loaded(bool loaded);
 
 private:
 	void button_toggled(GtkToggleButton* button);
@@ -70,7 +71,7 @@ private:
 	void show_about();
 	gboolean size_changed(XfcePanelPlugin*, gint size);
 	void update_size();
-	void show_menu(GtkWidget* parent, bool horizontal);
+	void show_menu(bool at_cursor);
 
 private:
 	XfcePanelPlugin* m_plugin;
