@@ -21,6 +21,7 @@
 #include "command.h"
 #include "favorites-page.h"
 #include "launcher-view.h"
+#include "plugin.h"
 #include "profile-picture.h"
 #include "recent-page.h"
 #include "resizer-widget.h"
@@ -39,12 +40,11 @@ using namespace WhiskerMenu;
 
 //-----------------------------------------------------------------------------
 
-#if GTK_CHECK_VERSION(3,20,0)
-
-static void grab_pointer(GdkWindow* window, guint32)
+static void grab_pointer(GtkWidget* widget)
 {
 	GdkDisplay* display = gdk_display_get_default();
 	GdkSeat* seat = gdk_display_get_default_seat(display);
+	GdkWindow* window = gtk_widget_get_window(widget);
 	gdk_seat_grab(seat, window, GDK_SEAT_CAPABILITY_ALL_POINTING, true, NULL, NULL, NULL, NULL);
 }
 
@@ -55,35 +55,13 @@ static void ungrab_pointer()
 	gdk_seat_ungrab(seat);
 }
 
-#else
-
-static void grab_pointer(GdkWindow* window, guint32 time)
-{
-	GdkDisplay* display = gdk_display_get_default();
-	GdkDeviceManager* device_manager = gdk_display_get_device_manager(display);
-	GdkDevice* device = gdk_device_manager_get_client_pointer(device_manager);
-	gdk_device_grab(device, window, GDK_OWNERSHIP_NONE, true,
-		GdkEventMask(
-			GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-			GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
-		),
-		NULL, time);
-}
-
-static void ungrab_pointer()
-{
-	GdkDisplay* display = gdk_display_get_default();
-	GdkDeviceManager* device_manager = gdk_display_get_device_manager(display);
-	GdkDevice* device = gdk_device_manager_get_client_pointer(device_manager);
-	gdk_device_ungrab(device, gtk_get_current_event_time());
-}
-
-#endif
-
 //-----------------------------------------------------------------------------
 
-WhiskerMenu::Window::Window() :
+WhiskerMenu::Window::Window(Plugin* plugin) :
+	m_plugin(plugin),
 	m_window(NULL),
+	m_search_cover(GTK_STACK_TRANSITION_TYPE_OVER_DOWN),
+	m_search_uncover(GTK_STACK_TRANSITION_TYPE_UNDER_UP),
 	m_sidebar_size_group(NULL),
 	m_layout_left(true),
 	m_layout_bottom(true),
@@ -123,6 +101,16 @@ WhiskerMenu::Window::Window() :
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
 	gtk_container_add(GTK_CONTAINER(m_window), frame);
 
+	// Create window contents stack
+	m_window_stack = GTK_STACK(gtk_stack_new());
+	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(m_window_stack));
+
+	// Create loading message
+	m_window_load_spinner = GTK_SPINNER(gtk_spinner_new());
+	gtk_widget_set_halign(GTK_WIDGET(m_window_load_spinner), GTK_ALIGN_CENTER);
+	gtk_widget_set_valign(GTK_WIDGET(m_window_load_spinner), GTK_ALIGN_CENTER);
+	gtk_stack_add_named(m_window_stack, GTK_WIDGET(m_window_load_spinner), "load");
+
 	// Create the profile picture
 	m_profilepic = new ProfilePicture(this);
 
@@ -189,8 +177,8 @@ WhiskerMenu::Window::Window() :
 
 	// Create box for packing children
 	m_vbox = GTK_BOX(gtk_box_new(GTK_ORIENTATION_VERTICAL, 6));
-	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(m_vbox));
 	gtk_container_set_border_width(GTK_CONTAINER(m_vbox), 2);
+	gtk_stack_add_named(m_window_stack, GTK_WIDGET(m_vbox), "contents");
 
 	// Create box for packing commands
 	m_commands_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
@@ -215,16 +203,18 @@ WhiskerMenu::Window::Window() :
 	gtk_box_pack_start(m_search_box, GTK_WIDGET(m_search_entry), true, true, 0);
 
 	// Create box for packing launcher pages and sidebar
+	m_contents_stack = GTK_STACK(gtk_stack_new());
 	m_contents_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6));
-	gtk_box_pack_start(m_vbox, GTK_WIDGET(m_contents_box), true, true, 0);
-	gtk_box_pack_start(m_contents_box, m_search_results->get_widget(), true, true, 0);
+	gtk_stack_add_named(m_contents_stack, GTK_WIDGET(m_contents_box), "contents");
+	gtk_stack_add_named(m_contents_stack, m_search_results->get_widget(), "search");
+	gtk_box_pack_start(m_vbox, GTK_WIDGET(m_contents_stack), true, true, 0);
 
 	// Create box for packing launcher pages
-	m_panels_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_VERTICAL, 0));
-	gtk_box_pack_start(m_contents_box, GTK_WIDGET(m_panels_box), true, true, 0);
-	gtk_box_pack_start(m_panels_box, m_favorites->get_widget(), true, true, 0);
-	gtk_box_pack_start(m_panels_box, m_recent->get_widget(), true, true, 0);
-	gtk_box_pack_start(m_panels_box, m_applications->get_widget(), true, true, 0);
+	m_panels_stack = GTK_STACK(gtk_stack_new());
+	gtk_box_pack_start(m_contents_box, GTK_WIDGET(m_panels_stack), true, true, 0);
+	gtk_stack_add_named(m_panels_stack, m_favorites->get_widget(), "favorites");
+	gtk_stack_add_named(m_panels_stack, m_recent->get_widget(), "recent");
+	gtk_stack_add_named(m_panels_stack, m_applications->get_widget(), "applications");
 
 	// Create box for packing sidebar
 	m_sidebar_buttons = GTK_BOX(gtk_box_new(GTK_ORIENTATION_VERTICAL, 0));
@@ -245,12 +235,7 @@ WhiskerMenu::Window::Window() :
 
 	// Show widgets
 	gtk_widget_show_all(frame);
-	gtk_widget_hide(m_favorites->get_widget());
-	gtk_widget_hide(m_recent->get_widget());
-	gtk_widget_hide(m_applications->get_widget());
-	gtk_widget_hide(m_search_results->get_widget());
 	m_default_button->set_active(true);
-	gtk_widget_show(frame);
 
 	// Resize to last known size
 	gtk_window_set_default_size(m_window, m_geometry.width, m_geometry.height);
@@ -307,16 +292,13 @@ void WhiskerMenu::Window::hide()
 	// Hide window
 	gtk_widget_hide(GTK_WIDGET(m_window));
 
-	// Reset mouse cursor by forcing default page to hide
-	gtk_widget_hide(m_default_page->get_widget());
-
 	// Switch back to default page
 	show_default_page();
 }
 
 //-----------------------------------------------------------------------------
 
-void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
+void WhiskerMenu::Window::show(const Position position)
 {
 	// Handle showing tooltips
 	if (wm_settings->launcher_show_tooltip)
@@ -344,13 +326,16 @@ void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
 	m_recent->enforce_item_count();
 
 	// Make sure applications list is current; does nothing unless list has changed
-	m_applications->load_applications();
-
-	// Focus search entry
-	gtk_widget_grab_focus(GTK_WIDGET(m_search_entry));
-
-	// Reset mouse cursor by forcing default page to hide
-	gtk_widget_show(m_default_page->get_widget());
+	if (m_applications->load_applications())
+	{
+		set_loaded();
+	}
+	else
+	{
+		m_plugin->set_loaded(false);
+		gtk_stack_set_visible_child_name(m_window_stack, "load");
+		gtk_spinner_start(m_window_load_spinner);
+	}
 
 	// Update default page
 	if (wm_settings->display_recent && (m_default_page == m_favorites))
@@ -377,10 +362,11 @@ void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
 
 	GdkScreen* screen = NULL;
 	int parent_x = 0, parent_y = 0, parent_w = 0, parent_h = 0;
-	if (parent != NULL)
+	if (position != PositionAtCursor)
 	{
 		// Wait up to half a second for auto-hidden panels to be shown
 		clock_t end = clock() + (CLOCKS_PER_SEC / 2);
+		GtkWidget* parent = m_plugin->get_button();
 		GtkWindow* parent_window = GTK_WINDOW(gtk_widget_get_toplevel(parent));
 		gtk_window_get_position(parent_window, &parent_x, &parent_y);
 		while ((parent_x == -9999) && (parent_y == -9999) && (clock() < end))
@@ -406,25 +392,15 @@ void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
 	else
 	{
 		GdkDisplay* display = gdk_display_get_default();
-#if GTK_CHECK_VERSION(3,20,0)
 		GdkSeat* seat = gdk_display_get_default_seat(display);
 		GdkDevice* device = gdk_seat_get_pointer(seat);
-#else
-		GdkDeviceManager* device_manager = gdk_display_get_device_manager(display);
-		GdkDevice* device = gdk_device_manager_get_client_pointer(device_manager);
-#endif
 		gdk_device_get_position(device, &screen, &parent_x, &parent_y);
 	}
 
 	// Fetch screen geomtry
 	GdkRectangle monitor;
-#if GTK_CHECK_VERSION(3,22,0)
 	GdkMonitor* monitor_gdk = gdk_display_get_monitor_at_point(gdk_display_get_default(), parent_x, parent_y);
 	gdk_monitor_get_geometry(monitor_gdk, &monitor);
-#else
-	int monitor_num = gdk_screen_get_monitor_at_point(screen, parent_x, parent_y);
-	gdk_screen_get_monitor_geometry(screen, monitor_num, &monitor);
-#endif
 
 	// Prevent window from being larger than screen
 	if (m_geometry.width > monitor.width)
@@ -441,7 +417,7 @@ void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
 	// Find window position
 	bool layout_left = ((2 * (parent_x - monitor.x)) + parent_w) < monitor.width;
 	bool layout_bottom = ((2 * (parent_y - monitor.y)) + (parent_h / 2)) > monitor.height;
-	if (horizontal)
+	if (position != PositionVertical)
 	{
 		m_geometry.x = layout_left ? parent_x : (parent_x + parent_w - m_geometry.width);
 		m_geometry.y = layout_bottom ? (parent_y - m_geometry.height) : (parent_y + parent_h);
@@ -519,13 +495,13 @@ void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
 	if ((layout_left && !wm_settings->position_categories_alternate)
 			|| (!layout_left && wm_settings->position_categories_alternate))
 	{
-		gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_panels_box), 1);
-		gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_sidebar), 2);
+		gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_panels_stack), 0);
+		gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_sidebar), 1);
 	}
 	else
 	{
-		gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_panels_box), 2);
-		gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_sidebar), 1);
+		gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_panels_stack), 1);
+		gtk_box_reorder_child(m_contents_box, GTK_WIDGET(m_sidebar), 0);
 	}
 	if (layout_left != m_layout_left)
 	{
@@ -603,26 +579,38 @@ void WhiskerMenu::Window::show(GtkWidget* parent, bool horizontal)
 		if (m_layout_bottom && m_layout_search_alternate)
 		{
 			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 0);
-			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_box), 1);
+			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_stack), 1);
 			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 2);
+
+			m_search_cover = GTK_STACK_TRANSITION_TYPE_OVER_UP;
+			m_search_uncover = GTK_STACK_TRANSITION_TYPE_UNDER_DOWN;
 		}
 		else if (m_layout_search_alternate)
 		{
 			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 2);
-			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_box), 1);
+			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_stack), 1);
 			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 0);
+
+			m_search_cover = GTK_STACK_TRANSITION_TYPE_OVER_DOWN;
+			m_search_uncover = GTK_STACK_TRANSITION_TYPE_UNDER_UP;
 		}
 		else if (m_layout_bottom)
 		{
 			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 0);
 			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 1);
-			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_box), 2);
+			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_stack), 2);
+
+			m_search_cover = GTK_STACK_TRANSITION_TYPE_OVER_DOWN;
+			m_search_uncover = GTK_STACK_TRANSITION_TYPE_UNDER_UP;
 		}
 		else
 		{
 			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 2);
 			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 1);
-			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_box), 0);
+			gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_stack), 0);
+
+			m_search_cover = GTK_STACK_TRANSITION_TYPE_OVER_UP;
+			m_search_uncover = GTK_STACK_TRANSITION_TYPE_UNDER_DOWN;
 		}
 	}
 
@@ -654,7 +642,7 @@ void WhiskerMenu::Window::save()
 
 void WhiskerMenu::Window::on_context_menu_destroyed()
 {
-	grab_pointer(gtk_widget_get_window(GTK_WIDGET(m_window)), gtk_get_current_event_time());
+	grab_pointer(GTK_WIDGET(m_window));
 }
 
 //-----------------------------------------------------------------------------
@@ -667,7 +655,6 @@ void WhiskerMenu::Window::set_categories(const std::vector<SectionButton*>& cate
 		gtk_box_pack_start(m_sidebar_buttons, GTK_WIDGET((*i)->get_button()), false, false, 0);
 		g_signal_connect_slot<GtkToggleButton*>((*i)->get_button(), "toggled", &Window::category_toggled, this);
 	}
-	gtk_widget_show_all(GTK_WIDGET(m_sidebar_buttons));
 
 	show_default_page();
 }
@@ -687,6 +674,21 @@ void WhiskerMenu::Window::set_items()
 
 //-----------------------------------------------------------------------------
 
+void WhiskerMenu::Window::set_loaded()
+{
+	// Hide loading spinner
+	gtk_spinner_stop(m_window_load_spinner);
+	gtk_stack_set_visible_child_name(m_window_stack, "contents");
+
+	// Focus search entry
+	gtk_widget_grab_focus(GTK_WIDGET(m_search_entry));
+
+	// Show panel button
+	m_plugin->set_loaded(true);
+}
+
+//-----------------------------------------------------------------------------
+
 void WhiskerMenu::Window::unset_items()
 {
 	m_search_results->unset_menu_items();
@@ -706,7 +708,7 @@ gboolean WhiskerMenu::Window::on_enter_notify_event(GtkWidget*, GdkEvent* event)
 		return false;
 	}
 
-	grab_pointer(gtk_widget_get_window(GTK_WIDGET(m_window)), crossing_event->time);
+	grab_pointer(GTK_WIDGET(m_window));
 
 	return false;
 }
@@ -722,7 +724,7 @@ gboolean WhiskerMenu::Window::on_leave_notify_event(GtkWidget*, GdkEvent* event)
 		return false;
 	}
 
-	grab_pointer(gtk_widget_get_window(GTK_WIDGET(m_window)), crossing_event->time);
+	grab_pointer(GTK_WIDGET(m_window));
 
 	return false;
 }
@@ -757,7 +759,7 @@ gboolean WhiskerMenu::Window::on_key_press_event(GtkWidget* widget, GdkEvent* ev
 	}
 
 	Page* page = NULL;
-	if (gtk_widget_get_visible(m_search_results->get_widget()))
+	if (gtk_stack_get_visible_child(m_contents_stack) == m_search_results->get_widget())
 	{
 		page = m_search_results;
 	}
@@ -827,7 +829,7 @@ gboolean WhiskerMenu::Window::on_map_event(GtkWidget*, GdkEvent*)
 	gtk_window_set_keep_above(m_window, true);
 
 	// Track mouse clicks outside of menu
-	grab_pointer(gtk_widget_get_window(GTK_WIDGET(m_window)), gtk_get_current_event_time());
+	grab_pointer(GTK_WIDGET(m_window));
 
 	// Focus search entry
 	gtk_widget_grab_focus(GTK_WIDGET(m_search_entry));
@@ -921,9 +923,7 @@ gboolean WhiskerMenu::Window::on_draw_event(GtkWidget* widget, cairo_t* cr)
 void WhiskerMenu::Window::favorites_toggled()
 {
 	m_favorites->reset_selection();
-	gtk_widget_hide(m_recent->get_widget());
-	gtk_widget_hide(m_applications->get_widget());
-	gtk_widget_show_all(m_favorites->get_widget());
+	gtk_stack_set_visible_child_name(m_panels_stack, "favorites");
 	gtk_widget_grab_focus(GTK_WIDGET(m_search_entry));
 }
 
@@ -932,9 +932,7 @@ void WhiskerMenu::Window::favorites_toggled()
 void WhiskerMenu::Window::recent_toggled()
 {
 	m_recent->reset_selection();
-	gtk_widget_hide(m_favorites->get_widget());
-	gtk_widget_hide(m_applications->get_widget());
-	gtk_widget_show_all(m_recent->get_widget());
+	gtk_stack_set_visible_child_name(m_panels_stack, "recent");
 	gtk_widget_grab_focus(GTK_WIDGET(m_search_entry));
 }
 
@@ -943,9 +941,7 @@ void WhiskerMenu::Window::recent_toggled()
 void WhiskerMenu::Window::category_toggled()
 {
 	m_applications->reset_selection();
-	gtk_widget_hide(m_favorites->get_widget());
-	gtk_widget_hide(m_recent->get_widget());
-	gtk_widget_show_all(m_applications->get_widget());
+	gtk_stack_set_visible_child_name(m_panels_stack, "applications");
 	gtk_widget_grab_focus(GTK_WIDGET(m_search_entry));
 }
 
@@ -992,16 +988,12 @@ void WhiskerMenu::Window::search()
 	if (visible)
 	{
 		// Show search results
-		gtk_widget_hide(GTK_WIDGET(m_sidebar));
-		gtk_widget_hide(GTK_WIDGET(m_panels_box));
-		gtk_widget_show(m_search_results->get_widget());
+		gtk_stack_set_visible_child_full(m_contents_stack, "search", m_search_cover);
 	}
 	else
 	{
 		// Show active panel
-		gtk_widget_hide(m_search_results->get_widget());
-		gtk_widget_show(GTK_WIDGET(m_panels_box));
-		gtk_widget_show(GTK_WIDGET(m_sidebar));
+		gtk_stack_set_visible_child_full(m_contents_stack, "contents", m_search_uncover);
 	}
 
 	// Apply filter
